<template>
  <div>
    <!--引用表格-->
    <BasicTable @register="registerTable" :rowSelection="rowSelection">
      <!--插槽:table标题-->
      <template #tableTitle>
        <a-button type="primary" preIcon="ant-design:plus-outlined" @click="handleCreate"> 新增</a-button>
        <a-button type="primary" preIcon="ant-design:export-outlined" @click="onExportXls"> 导出</a-button>
        <j-upload-button type="primary" preIcon="ant-design:import-outlined" @click="onImportXls">导入</j-upload-button>
        <a-button type="primary" @click="handleSyncUser" preIcon="ant-design:sync-outlined"> 同步流程</a-button>
        <a-button type="primary" @click="openModal(true, {})" preIcon="ant-design:hdd-outlined"> 回收站</a-button>
        <JThirdAppButton biz-type="user" :selected-row-keys="selectedRowKeys" syncToApp syncToLocal
          @sync-finally="onSyncFinally" />
        <a-dropdown v-if="selectedRowKeys.length > 0">
          <template #overlay>
            <a-menu>
              <a-menu-item key="1" @click="batchHandleDelete">
                <Icon icon="ant-design:delete-outlined" />
                删除
              </a-menu-item>
              <a-menu-item key="2" @click="batchFrozen(2)">
                <Icon icon="ant-design:lock-outlined" />
                冻结
              </a-menu-item>
              <a-menu-item key="3" @click="batchFrozen(1)">
                <Icon icon="ant-design:unlock-outlined" />
                解冻
              </a-menu-item>
            </a-menu>
          </template>
          <a-button>批量操作
            <Icon icon="mdi:chevron-down" />
          </a-button>
        </a-dropdown>
      </template>
      <!--操作栏-->
      <template #action="{ record }">
        <TableAction :actions="getTableAction(record)" :dropDownActions="getDropDownAction(record)" />
      </template>
    </BasicTable>
    <!--用户抽屉-->
    <UserDrawer @register="registerDrawer" @success="handleSuccess" />
    <!--修改密码-->
    <PasswordModal @register="registerPasswordModal" @success="reload" />
    <!--用户代理-->
    <UserAgentModal @register="registerAgentModal" @success="reload" />
    <!--回收站-->
    <UserRecycleBinModal @register="registerModal" @success="reload" />
    <!-- 离职受理人弹窗 -->
    <UserQuitAgentModal @register="registerQuitAgentModal" @success="reload" />
    <!-- 离职人员列弹窗 -->
    <UserQuitModal @register="registerQuitModal" @success="reload" />
  </div>
</template>

<script lang="ts" name="system-user" setup>
//ts语法
import { unref } from 'vue'
import PasswordModal from './PasswordModal.vue'
import { batchDeleteUser, deleteUser, frozenBatch, getExportUrl, getImportUrl, listNoCareTenant, syncUser } from './user.api'
import { columns, searchFormSchema } from './user.data'
import UserAgentModal from './UserAgentModal.vue'
import UserDrawer from './UserDrawer.vue'
import UserQuitAgentModal from './UserQuitAgentModal.vue'
import UserQuitModal from './UserQuitModal.vue'
import UserRecycleBinModal from './UserRecycleBinModal.vue'
import { useDrawer } from '/@/components/Drawer'
import JThirdAppButton from '/@/components/jeecg/thirdApp/JThirdAppButton.vue'
import { useModal } from '/@/components/Modal'
import { ActionItem, BasicTable, TableAction } from '/@/components/Table'
import { useListPage } from '/@/hooks/system/useListPage'
import { useMessage } from '/@/hooks/web/useMessage'
import { usePermission } from '/@/hooks/web/usePermission'

const { createMessage, createConfirm } = useMessage()
const { isDisabledAuth } = usePermission()
//注册drawer
const [registerDrawer, { openDrawer }] = useDrawer()
//回收站model
const [registerModal, { openModal }] = useModal()
//密码model
const [registerPasswordModal, { openModal: openPasswordModal }] = useModal()
//代理人model
const [registerAgentModal, { openModal: openAgentModal }] = useModal()
//离职代理人model
const [registerQuitAgentModal, { openModal: openQuitAgentModal }] = useModal()
//离职用户列表model
const [registerQuitModal, { openModal: openQuitModal }] = useModal()

// 列表页面公共参数、方法
const { prefixCls, tableContext, onExportXls, onImportXls } = useListPage({
  designScope: 'user-list',
  tableProps: {
    title: '用户列表',
    api: listNoCareTenant,
    columns: columns,
    size: 'small',
    formConfig: {
      // labelWidth: 200,
      schemas: searchFormSchema,
    },
    actionColumn: {
      width: 120,
    },
    beforeFetch: (params) => {
      return Object.assign({ column: 'createTime', order: 'desc' }, params)
    },
  },
  exportConfig: {
    name: '用户列表',
    url: getExportUrl,
  },
  importConfig: {
    url: getImportUrl,
  },
})

//注册table数据
const [registerTable, { reload, updateTableDataRecord }, { rowSelection, selectedRows, selectedRowKeys }] = tableContext

/**
 * 新增事件
 */
function handleCreate() {
  openDrawer(true, {
    isUpdate: false,
    showFooter: true,
  })
}
/**
 * 编辑事件
 */
async function handleEdit(record: Recordable) {
  openDrawer(true, {
    record,
    isUpdate: true,
    showFooter: true,
  })
}
/**
 * 详情
 */
async function handleDetail(record: Recordable) {
  openDrawer(true, {
    record,
    isUpdate: true,
    showFooter: false,
  })
}
/**
 * 删除事件
 */
async function handleDelete(record) {
  if ('admin' == record.username) {
    createMessage.warning('管理员账号不允许此操作！')
    return
  }
  await deleteUser({ id: record.id }, reload)
}
/**
 * 批量删除事件
 */
async function batchHandleDelete() {
  let hasAdmin = unref(selectedRows).filter((item) => item.username == 'admin')
  if (unref(hasAdmin).length > 0) {
    createMessage.warning('管理员账号不允许此操作！')
    return
  }
  await batchDeleteUser({ ids: selectedRowKeys.value }, () => {
    selectedRowKeys.value = []
    reload()
  })
}
/**
 * 成功回调
 */
function handleSuccess() {
  reload()
}

/**
 * 打开修改密码弹窗
 */
function handleChangePassword(username) {
  openPasswordModal(true, { username })
}
/**
 * 打开代理人弹窗
 */
function handleAgentSettings(userName) {
  openAgentModal(true, { userName })
}
/**
 * 冻结解冻
 */
async function handleFrozen(record, status) {
  if ('admin' == record.username) {
    createMessage.warning('管理员账号不允许此操作！')
    return
  }
  await frozenBatch({ ids: record.id, status: status }, reload)
}
/**
 * 批量冻结解冻
 */
function batchFrozen(status) {
  let hasAdmin = selectedRows.value.filter((item) => item.username == 'admin')
  if (unref(hasAdmin).length > 0) {
    createMessage.warning('管理员账号不允许此操作！')
    return
  }
  createConfirm({
    iconType: 'warning',
    title: '确认操作',
    content: '是否' + (status == 1 ? '解冻' : '冻结') + '选中账号?',
    onOk: async () => {
      await frozenBatch({ ids: unref(selectedRowKeys).join(','), status: status }, reload)
    },
  })
}

/**
 *同步流程
 */
function handleSyncUser() {
  syncUser()
}
/**
 *同步钉钉和微信回调
 */
function onSyncFinally({ isToLocal }) {
  // 同步到本地时刷新下数据
  if (isToLocal) {
    reload()
  }
}

/**
 * 操作栏
 */
function getTableAction(record): ActionItem[] {
  return [
    {
      label: '编辑',
      onClick: handleEdit.bind(null, record),
      // ifShow: () => hasPermission('system:user:edit'),
    },
  ]
}
/**
 * 下拉操作栏
 */
function getDropDownAction(record): ActionItem[] {
  return [
    {
      label: '详情',
      onClick: handleDetail.bind(null, record),
    },
    {
      label: '密码',
      //auth: 'user:changepwd',
      onClick: handleChangePassword.bind(null, record.username),
    },
    {
      label: '删除',
      popConfirm: {
        title: '是否确认删除',
        confirm: handleDelete.bind(null, record),
      },
    },
    {
      label: '冻结',
      ifShow: record.status == 1,
      popConfirm: {
        title: '确定冻结吗?',
        confirm: handleFrozen.bind(null, record, 2),
      },
    },
    {
      label: '解冻',
      ifShow: record.status == 2,
      popConfirm: {
        title: '确定解冻吗?',
        confirm: handleFrozen.bind(null, record, 1),
      },
    },
    {
      label: '代理人',
      onClick: handleAgentSettings.bind(null, record.username),
    },
  ]
}

/**
 * 离职
 * @param userName
 */
function handleQuit(userName) {
  //打开离职代理人弹窗
  openQuitAgentModal(true, { userName })
}
</script>

<style scoped></style>
